/*
  Copyright (C) 2001 Jesper Hansen <jesperh@telia.com>.

  Written by:  Romuald Bialy (aka MIS) <romek_b@o2.pl>

  Some ideas from Nikolai Vorontsov <nickviz@mail.be>

  This file is part of the yampp system.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <io.h>
#include <progmem.h>
#include <string.h>

#include "mem.h"
#include "uart.h"
#include "atapi.h"
#include "iso9660.h"
#include "lcd.h"
#include "delay.h"

//Global variables

bool joliet;
u32  dwFileSize;		// Current file size
u32  dwBytesLoaded;		// Counter of bytes loaded from CD
bool bBuf1Empty	= true;		// Empty flags for buffers
bool bBuf2Empty	= true;
u08  nLastPosD = 0;		// Directory tabele last element number
u08  nLastEntry = 0;		// For directory change checking
u32  dwCurrentBlock;		// Current being read block

static u32 PathBlock;		// Block number for Path Tabele
static u32 dwDirInCache;
static u08 PathSize;		// Size of Path Tabele in blocks
static u08 *dst;		// Temp var, do not need to be in stack
static u08 nCurEntry;		// Couner of current dir entry during get_de() 					
static u08 dirlisting = 0;	// Mode of diectory browsing 
static u08 nNewEntry;

bool FindBootBlock(u32 sig);

//============================================================================================
// Get inserted CD toc and name
//============================================================================================
bool ISO_INIT(void)
{
	u08 i;

	*(u16*)(LONG_NAME - 2) = 0xFFFF;			// This are special 3 bytes, 
	*(LONG_NAME - 3) = 0xFF;				//  placed to help in .mp3
 								//  extension checking
	*CD_NAME = 0;
	CD_Cmd(START_CD);
	delayms(500);
	joliet = true;
	if (!FindBootBlock(0x30444302))				// Check secondary Boot Block
	{
		joliet = false;					// Wrong CD
		if (!FindBootBlock(0x30444301)) return false;	// Check primary Boot Block
	}
	dst = CD_NAME;
	for( i=joliet ; i < ISO_VOL_ID_LEN; i += joliet+1 )
	{
		*dst = BUFFER1P[ISO_VOL_ID + i];		// Copy CD Name
		if(*(dst++) == 0) break;
	}
	*dst = 0;
	lcd_clrscr();
	lcd_puts(CD_NAME);
	PRINT_p(PSTR("Found "));
	if (joliet)
	{
		lcd_puts_p(PSTR("\nJoliet disc."));
		PRINT_p(PSTR("Joliet disc: ")); 
	}
	else
	{
		lcd_puts_p(PSTR("\nISO9660 disc."));
		PRINT_p(PSTR("ISO9660 disc: "));
	}
	PRINT(CD_NAME); EOL();
	PathBlock = *(u32*)(BUFFER1P + ISO_PTABLE);			// Location of Path Tabele
	PathSize  = (*(u32*)(BUFFER1P + ISO_PTABLE - 8)) / BPS +1;	// Size of Path Tabele
	if (PathSize>2) PathSize=2;					// no more ram to load bigger PT
	return true;
}	

bool FindBootBlock(u32 sig)
{
	u08 i;
	for(i=0; i<5; i++)
	{
		while(ATAPI_ReadLBA(ISO_BOOT_BLOCK + i , BUFFER1P));
		if(*(u32*)BUFFER1P == sig)
			return true;
	}
	return false;
}


//============================================================================================
// Search for files inside the directory
//============================================================================================
bool GetFileList(u32 adr)
{
	register u08 i;
	u16 offset;
	struct direntry *de;
	u08 nStartEntry = nCurEntry;					// Save first entry number in a directory
	bool block_rd = false;

LD_SEC:	offset = 0;
	if (adr != dwDirInCache)
	{
		while(ATAPI_ReadLBA(adr, DIR_CACHE));
		dwDirInCache = adr;
		block_rd = true;
	}
	do
	{
		*LONG_NAME = 0;
		de = (struct direntry *)(DIR_CACHE + offset);
		if ((de->Flags & (Flag_NotExist | Flag_Directory)) == 0)	// Normal file
		{
			dst = LONG_NAME;
			for(i=joliet; i < (de->Len_Fi - 2) ; i += (joliet+1))
			{
				*dst = de->FileName[i];
				if (*dst != ';') dst++;
			}
			*dst = 0;	
			if (*--dst == '3' && (*--dst == 'p'  ||  *dst == 'P')  &&	// if MP3 file
			   (*--dst == 'm'  ||  *dst == 'M') &&	*--dst == '.')

			{
				*dst = 0;					// cut it
				if (dirlisting == DIRLIST_VERBOSE)	
				{						// print the name
					UART_Puts(DIR_NAME);
					PRINT_p(PSTR(" / "));
					UART_Putsln(LONG_NAME);
				}
				// If this is the entry we're looking for, exit
				if (nCurEntry == nNewEntry)
				{
					//dwBytesLoaded = 0;
					dwCurrentBlock = de->FileLocM;
					dwFileSize = de->FileLenM;
					return true;
				}
				if(block_rd && dirlisting && (nLastPosD < MAX_DIRS))	
										// This is first mp3 file in directory
				{
					block_rd = false;
					DirsTable[nLastPosD].dir_start = (nStartEntry == nCurEntry);
					DirsTable[nLastPosD].song_num = nCurEntry; 
										// first file in dir for dir navigation
					DirsTable[nLastPosD++].dir_block = adr; 
										// dir Block number for fast load
				//	UART_Printfu08(nCurEntry); UART_SendByte(' '); UART_Printfu32(adr); EOL();  // Debug
				}
				nCurEntry++;		// Increase song number
			} // if MP3 file
		} // if Normal file		
		offset += de->LEN_DR;
	} while ((de->LEN_DR > 0) && ((u08)de->FileLocL == 0));		// Check for an empty or not directory record
	if (offset > 1900)						// Continue possible in next block
	{
		adr++;							// Next block addres
		goto LD_SEC;						// Check next block
	}
	else
		return false;						// Reached end of directory, track not found
}

//============================================================================================
// Read Path Tabele and call to GetFileList on all found directory
//============================================================================================
bool get_de(void)

{
	register u08 i;
	u16 offset = 0;
	struct pathtab *pt = 0;
	while(ATAPI_ReadLBA(PathBlock, BUFFER1P));				// Read Path Tabele to Buffer1
	if (PathSize == 2)							// if more than 1 block
		while(ATAPI_ReadLBA(PathBlock + 1, BUFFER1P + BPS));		// Read rest of PT
	while(1)
	{
		*DIR_NAME = 0;
		pt = (struct pathtab *)(BUFFER1P + offset);			// Adres of current PT record
		if (pt->DirNameLen > 0)
		{
			dst = DIR_NAME;
			for(i=joliet; i < pt->DirNameLen; i += (joliet+1))	// Get Dir name
				*dst++ = pt->DirName[i];
			*dst = 0;
#if (LCD_LINES == 2)
			strcat(DIR_NAME," / ");
#else
			if(strlen(DIR_NAME) > LCD_LINE_LENGTH)
				strcat(DIR_NAME," >> ");
#endif
			if (GetFileList(pt->DirLoc))				// Get File list inside directory
				return true;					// Return if found entry
			offset += (8 + pt->DirNameLen);				// Calculate next PT record offset
			if (pt->DirNameLen % 2  == 1) 				// Skip padding field if DirNameLen
				offset++;					// is an odd number
		}
		else
			return false;	
	}
}

//============================================================================================
// Initial read CD contents
//============================================================================================
u08 dirlist(u08 mode)
{
	dirlisting = mode;
	nLastPosD = 0;			// Directory tabele last element number = 0
	nCurEntry = 0;			// current entry number = 0
	nNewEntry = 255;
	get_de();			// scan through all
	dirlisting = 0;
	return nCurEntry;
}


//============================================================================================
// This function sets information about MP3 file number 'entry'
// Returns true in file was found and false otherwise
//============================================================================================
bool get_dir_entry(u08 entry)
{
register u08 i = 0;

	nNewEntry = entry;
	for (i = 1; i < nLastPosD; i++)
		if (DirsTable[i].song_num > entry) break;	// search for dir number of selected file
	i--;
	if (nLastEntry == i)					// check for dir change
	{							// song in the same dir
		nCurEntry = DirsTable[i].song_num;		// set current entry at first file in dir
#if (LCD_LINES == 2)
		dst = strchr(DIR_NAME,'/');			// check for slash in directory name
		if(dst)						// if slash found
			*(dst+2) = 0;				// cut file name from dir string
		else						// slash not found, it's root directory
			*DIR_NAME = 0;				// clear old name
#endif
		return GetFileList(DirsTable[i].dir_block);	// get directory from current dir block
	}
	else	
	{							// song in different dir
		nCurEntry = 0;
		nLastEntry = i;					// save actual dir number for directory change check
		return get_de();				// search from root directory
	}
}


extern bool bError;
//============================================================================================
// Check for more avaliable data to load, or error occurence
//============================================================================================
bool CheckAndAdvanceBlock(void)
{
	return ((dwBytesLoaded <= dwFileSize - BUFFER_SIZE) && (!bError));
}

//============================================================================================
// Read one block from CD to one of buffer BUF1 or BUF2
//============================================================================================
bool ReadBuffer(u08 nBuf)
{
	register u08* pBuf;
	if (nBuf == BUF1)
	{
		pBuf = BUFFER1P;
		bBuf1Empty = false;
	}
	else
	{
		pBuf = BUFFER2P;
		bBuf2Empty = false;
	}
	while((bError = ATAPI_ReadLBA(dwCurrentBlock, pBuf)) > ERR_NOCD);
	if (! bError)
	{
		dwCurrentBlock += BUFFER_SIZE / BPS;
		dwBytesLoaded += BUFFER_SIZE;
	}
	return bError;
}
