/* ***********************************************************************
**
**  Copyright (C) 2002 	Romuald Bialy (MIS) <romek_b@o2.pl>.
**
**
**  Yampp-3/USB - Constants and options definition file
**
**  File Constants.c
**
*************************************************************************
**
**   This file is part of the yampp system.
**
**  This program is free software; you can redistribute it and/or
**  modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2
**  of the License, or (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software Foundation, 
**  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**
*************************************************************************
**
**  Revision History
**
**  when         what  who   why
**
**  2002-09-01   1.0   MIS   initial public release
**
*********************************************************************** */

// Overall current firmware version is 1.00

#ifndef __CONST_H__
#define __CONST_H__

// Here is some constants are defined.

//============================================================================
// VS1001
//============================================================================
//
// Define F_VS1001 constant according to your VS1001 clock divided by 2000.
// Add 0x8000 if you use clock doubler.
#define F_VS1001	(6144 + 0x8000)			/* 12,288MHz x2 */
//#define F_VS1001	(12288 + 0x0000)		/* 24.576Mhz */


//============================================================================
// DEFINITION OF LCD ORGANIZATION
//============================================================================
//
//#define LCD_TYPE 	1		// 2x16 LCD
//#define LCD_TYPE 	2		// 2x20 LCD
//#define LCD_TYPE 	3		// 2x24 LCD
//#define LCD_TYPE 	4		// 2x40 LCD
//#define LCD_TYPE 	5		// 4x16 LCD
#define LCD_TYPE 	6		// 4x20 LCD


//============================================================================
// HDD OPTIONS
//============================================================================
//
// Enter here time (in seconds) to spindown yampp HDD after STOP command 
// (0 = dont'stop, 1200 max = about 20 minutes)
#define HDD_STANDBY_TIME 15
//----------------------------------------------------------------------------
// In case your disk very fast enter in idle mode and you hear sound dropout - 
// uncomment the following line
//#define FAST_IDLE_DISK


//============================================================================
// UART OPTIONS
//============================================================================
//
// In case you need uart progressbar - uncomment the following line
#define UART_PROGRESSBAR
//----------------------------------------------------------------------------
// In case you need uart song time displaying - uncomment the following line
//#define UART_SONGTIME


//============================================================================
// Infra Red Controls
//============================================================================
//
// In case you have a problems with volume keys - comment the following line.
#define ENABLE_AUTOREPEAT
//----------------------------------------------------------------------------
// In case you have a problems with double interpretted random, loudness
// and numerical keys - set following constants value to 2,3 or 4.
// Note:  Increase this value slow down AUTOREPEAT function too.
#define DOUBLE_TRAP 1


//============================================================================
// Local keys control and layout definitions
//============================================================================
//
// If you need player control only from four local keys - uncomment the
// following line
//#define FOUR_KEYS_CONTROL
//----------------------------------------------------------------------------
//
#ifndef FOUR_KEYS_CONTROL
//
// Here is local keys layout definitions for 8 keys control
//
// Shortpress
#define KBD_SHORT { EV_IDLE, EV_PLAY, EV_STOP,     EV_NEXT, EV_PREV, EV_NEXTPL, EV_PREVPL, EV_UP, EV_DOWN };
// Longpress
#define KBD_LONG  { EV_IDLE, EV_MENU, EV_PLAYLIST, EV_FFWD, EV_FREW, EV_RANDOM, EV_REPEAT, EV_UP, EV_DOWN };
//
//
#else
//
//
// Here is local keys layout definitions for 4 keys control
//
// Shortpress
#define KBD_SHORT { EV_IDLE, EV_PLAY, EV_STOP,     EV_UP, EV_DOWN, EV_IDLE,EV_IDLE,EV_IDLE,EV_IDLE};
// Longpress
#define KBD_LONG  { EV_IDLE, EV_MENU, EV_PLAYLIST, EV_UP, EV_DOWN, EV_IDLE,EV_IDLE,EV_IDLE,EV_IDLE};
//
//
#endif


//============================================================================
// APPLICATION
//============================================================================
//
// This constant defines minimum volume level - to decrease unnessesary
// steps in volume control
// note that 0 is max volume and 0xff is minimum volume
#define MIN_VOLUME 	0x50
//----------------------------------------------------------------------------
// Number of volume steps 
#define NUM_VOL_STEPS	24
//----------------------------------------------------------------------------
//In case you can hear three power up beeps - uncomment the following line:
//#define PWR_BEEPS


//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
//
// Some checks and calculations - DON'T change it !
//

#define VOL_STEP	( (MIN_VOLUME + 1) / NUM_VOL_STEPS )

#if HDD_STANDBY_TIME > 1200
#error Invalid HDD_STANDBY_TIME !
#endif

#if F_VS1001 < 10000 || F_VS1001 > 65536
#error Invalid F_VS1001 !
#endif

#if (LCD_TYPE == 1)
 #define LCD_LINES 2
 #define LCD_LINE_LENGTH  16
#endif
#if (LCD_TYPE == 2)
 #define LCD_LINES 2
 #define LCD_LINE_LENGTH  20
#endif
#if (LCD_TYPE == 3)
 #define LCD_LINES 2
 #define LCD_LINE_LENGTH  24
#endif
#if (LCD_TYPE == 4)
 #define LCD_LINES 2
 #define LCD_LINE_LENGTH  40
#endif
#if (LCD_TYPE == 5)
 #define LCD_LINES 4
 #define LCD_LINE_LENGTH  16
#endif
#if (LCD_TYPE == 6)
 #define LCD_LINES 4
 #define LCD_LINE_LENGTH  20
#endif

#endif // __CONST_H__
