/* ***********************************************************************
**
**  Copyright (C) 2002  Jesper Hansen <jesperh@telia.com> and 
**			Romuald Bialy (MIS) <romek_b@o2.pl>.
**
**
**  Yampp-3/USB - main header file
**
**  File yampp3_usb.h
**
*************************************************************************
**
**   This file is part of the yampp system.
**
**  This program is free software; you can redistribute it and/or
**  modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2
**  of the License, or (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software Foundation, 
**  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**
*************************************************************************
**
**  Revision History
**
**  when         what  who	why
**
**  2002-09-01   1.0   MIS	initial public release
**
*********************************************************************** */

#include <io.h>
#include <progmem.h>
#include <eeprom.h>
#include <sig-avr.h>
#include <interrupt.h>
#include <wdt.h>
#include <string.h>
#include <stdarg.h>

#include "types.h"
#include "Constants.h"
#include "yampp3lib.h"
#include "mem.h"
#include "printf.h"
#include "yadl.h"
#include "usb.h"

u08 *SectorBuffer;
extern u08 *updbuf;
extern u08 *buffer1;

u08 nKeyTime;
u08 ir_standard;
u08 usb_link;

typedef enum
{	
	EV_IDLE,
	EV_PLAY,
	EV_STOP,
	EV_NEXT,
	EV_PREV,
	EV_NEXTPL,	// next playlist
	EV_PREVPL,	// prevous playlist
	EV_FFWD,
	EV_FREW,
	EV_MBASS,	// old loudness
	EV_RANDOM,	// random on/off
	EV_REPEAT,	// repeat mode
	EV_TIME,	// time display (normal <-> remain)
	EV_UP,		// volume or search UP
	EV_DOWN,	// volume or search DOWN
	EV_PLAYLIST,	// browse playlists
	EV_MENU,	// Not work yet !!!
	EV_NEUTRAL,	// volume to -12dB
	EV_PLAY_USB,	// playing via USB
	EV_REMOTE_CFG,	// setup remote codes
	EV_LASTEVENT	// for events count calculation
} event_e;	

#define STANDARDS_COUNT 5	// number of IR remote standards

event_e get_event(void);
void set_event(event_e e);
void send_sinewave_beeps(void);
void eeprom_ww(u16 address, u16 value);
void lcd_clrline(u08 line);

extern void control_handler(void);
extern u08 rec80_active(void);
extern u16 get_rec80(u08 std);
extern void setup_rem(void);
